import asyncio
from io import BytesIO
from typing import Any

from fortnite_api import Client
from fortnite_api.enums import StatsImageType, TimeWindow
from fortnite_api.errors import FortniteAPIException
import httpx
from PIL import Image, ImageDraw, ImageFont

from .config import CHINESE_FONT_PATH, fconfig

API_KEY: str | None = fconfig.fortnite_api_key


def handle_fortnite_api_exception(e: FortniteAPIException) -> str:
    err_msg = str(e)
    if "public" in err_msg:
        return "战绩未公开"
    elif "exist" in err_msg:
        return "用户不存在"
    elif "match" in err_msg:
        return "该玩家当前赛季没有进行过任何对局"
    elif "timed out" in err_msg:
        return "请求超时, 请稍后再试"
    elif "failed to fetch" in err_msg:
        return "拉取账户信息失败, 稍后再试"
    else:
        return f"未知错误: {err_msg}"


async def get_level(name: str, cmd_header: str) -> str:
    time_window: Any = TimeWindow.LIFETIME if cmd_header.startswith("生涯") else TimeWindow.SEASON
    try:
        async with Client(api_key=API_KEY) as client:
            stats = await client.fetch_br_stats(name=name, time_window=time_window)
    except FortniteAPIException as e:
        return handle_fortnite_api_exception(e)
    bp = stats.battle_pass
    if bp is None:
        return f"未查询到 {stats.user.name} 的季卡等级"
    return f"{stats.user.name}: Lv{bp.level} | {bp.progress}% to Lv{bp.level + 1}"


async def get_stats_image(name: str, cmd_header: str) -> BytesIO:
    time_window: Any = TimeWindow.LIFETIME if cmd_header.startswith("生涯") else TimeWindow.SEASON
    image_type: Any = StatsImageType.ALL
    try:
        async with Client(api_key=API_KEY) as client:
            stats = await client.fetch_br_stats(
                name=name,
                time_window=time_window,
                image=image_type,
            )
    except FortniteAPIException as e:
        raise ValueError(handle_fortnite_api_exception(e))
    if stats.image is None:
        raise ValueError(f"未查询到 {stats.user.name} 的战绩")
    return await get_stats_img_by_url(stats.image.url, stats.user.name)


async def get_stats_img_by_url(url: str, name: str) -> BytesIO:
    async with httpx.AsyncClient(verify=False, timeout=15) as client:
        # 发送GET请求获取图片数据
        response = await client.get(url)

        # 检查请求是否成功
        if response.status_code != 200:
            raise ValueError(f"无法获取图片, 状态码: {response.status_code}")

        # 将响应内容转换为字节流
        image_data = BytesIO(response.content)
    # 如果不包含中文名，返回原图
    if not contains_chinese(name):
        return image_data

    return await process_image_with_chinese(image_data, name)


def contains_chinese(text: str) -> bool:
    import re

    pattern = re.compile(r"[\u4e00-\u9fff]")
    return bool(pattern.search(text))


async def process_image_with_chinese(file: BytesIO, name: str) -> BytesIO:
    return await asyncio.to_thread(_process_image_with_chinese, file, name)


from functools import lru_cache

# @lru_cache(maxsize=1)
# def create_gradient_image(width: int = 397, height: int = 140) -> Image.Image:
#     import numpy as np

#     # 创建渐变图像
#     gradient = np.zeros((height, width, 3), dtype=np.uint8)
#     start_color = np.array([0, 33, 69])
#     end_color = np.array([0, 82, 106])

#     # 向量化计算渐变
#     for i in range(width):
#         for j in range(height):
#             ratio = (i + j) / (width + height)
#             gradient[j, i] = start_color + (end_color - start_color) * ratio

#     # 将渐变图像粘贴到原图
#     return Image.fromarray(gradient)


@lru_cache(maxsize=1)
def create_gradient_image_new() -> Image.Image:
    """从底图裁剪渐变图片"""
    from .config import STATS_BG_PATH

    left, top, right, bottom = 26, 90, 423, 230

    with Image.open(STATS_BG_PATH) as img:
        gradient_img = img.crop((left, top, right, bottom))
        return gradient_img


def _process_image_with_chinese(bytes_io: BytesIO, name: str) -> BytesIO:
    with Image.open(bytes_io, formats=["PNG"]) as img:
        draw = ImageDraw.Draw(img)

        # 矩形区域的坐标
        left, top, right, bottom = 26, 90, 423, 230

        # 创建渐变色并填充矩形区域
        # width = right - left, height = bottom - top
        gradient_img = create_gradient_image_new()
        img.paste(gradient_img, (left, top))
        # 指定字体
        font_size = 36
        font = ImageFont.truetype(CHINESE_FONT_PATH, font_size)

        # 计算字体坐标
        length = draw.textlength(name, font=font)
        x = left + (right - left - length) / 2
        y = top + (bottom - top - font_size) / 2
        draw.text((x, y), name, fill="#fafafa", font=font)

        output_bytes = BytesIO()
        # 保存处理后的图像到 BytesIO
        img.save(output_bytes, format="PNG", optimize=True)
        # 将指针重置到 BytesIO 对象的开头
        output_bytes.seek(0)

        return output_bytes



from pathlib import Path

from nonebot import get_plugin_config
import nonebot_plugin_localstore as store
from pydantic import BaseModel


class Config(BaseModel):
    fortnite_api_key: str | None = None


fconfig: Config = get_plugin_config(Config)

cache_dir: Path = store.get_plugin_cache_dir()
data_dir: Path = store.get_plugin_data_dir()

CHINESE_FONT_PATH: Path = data_dir / "SourceHanSansSC-Bold-2.otf"
GG_FONT_PATH: Path = data_dir / "burbankbigregular-black.woff2"
VB_FONT_PATH: Path = data_dir / "LuckiestGuy.woff"
STATS_BG_PATH: Path = data_dir / "stats_bg.png"



import asyncio
from pathlib import Path
import time

from nonebot.log import logger
from nonebot_plugin_htmlrender import get_new_page
from nonebot_plugin_htmlrender.browser import Page
from PIL import Image, ImageDraw, ImageFont
from playwright.async_api import Locator, Route

from .config import VB_FONT_PATH, cache_dir, data_dir

vb_file = data_dir / "vb.png"
hot_info_1_path = cache_dir / "hot_info_1.png"
container_hidden_xs_path = cache_dir / "container_hidden_xs.png"
hot_info_2_path = cache_dir / "hot_info_2.png"


async def screenshot_vb_img() -> Path:
    async with get_new_page(device_scale_factor=1) as page:
        await _screenshot_vb_img(page)
    await combine_imgs()
    return vb_file


async def _screenshot_vb_img(page: Page):
    url = "https://freethevbucks.com/timed-missions"

    # 拦截广告
    async def ad_block_handler(route: Route):
        ad_domains = [
            "googlesyndication.com",
            "doubleclick.net",
            "adnxs.com",
            "google-analytics.com",
            "facebook.com",
            "amazon-adsystem.com",
            "adform.net",
            "googleadservices.com",
            "doubleclick.net",
        ]
        if any(ad_domain in route.request.url for ad_domain in ad_domains):
            await route.abort()
        else:
            await route.continue_()

    await page.route("**/*", ad_block_handler)

    await page.goto(url)

    # 截图函数，超时则跳过
    async def take_screenshot(locator: Locator, path: Path) -> None:
        try:
            # 检查元素内容是否为空
            content = await locator.inner_html()
            if content.strip():
                await asyncio.wait_for(locator.screenshot(path=path), timeout=5)
            else:
                logger.warning(f"Locator for {path.name} is empty.")
        except Exception:
            pass

    # 截取第一个 <div class="hot-info">
    hot_info_1 = page.locator("div.hot-info").nth(0)
    await take_screenshot(hot_info_1, hot_info_1_path)

    # 截取 <div class="container hidden-xs">
    container_hidden_xs = page.locator("div.container.hidden-xs")
    await take_screenshot(container_hidden_xs, container_hidden_xs_path)

    # 截取第二个 <div class="hot-info">
    hot_info_2 = page.locator("div.hot-info").nth(1)
    await take_screenshot(hot_info_2, hot_info_2_path)


async def combine_imgs():
    await asyncio.to_thread(_combine_imgs)


def _combine_imgs():
    # 打开截图文件（如果存在）
    img_paths = [hot_info_1_path, container_hidden_xs_path, hot_info_2_path]
    img_paths = [i for i in img_paths if i.exists()]
    if not img_paths:
        raise Exception("所有选择器的截图文件均不存在")
    # 先添加时间
    try:
        # images = [Image.open(img_path) for img_path in img_paths]
        with (
            Image.open(img_paths[0]) as img1,
            Image.open(img_paths[1]) as img2,
            Image.open(img_paths[2]) as img3,
        ):
            images: list[Image.Image] = [img1, img2, img3]

            # 获取尺寸并创建新图像
            widths, heights = zip(*(img.size for img in images))
            total_width = max(widths)
            total_height = sum(heights)

            # 如果 img1.width < total_width，则拉伸最右侧像素到 total_width
            if img1.width < total_width:
                img1 = resize_img_with_right_pixel(img1, total_width)
                images[0] = img1

            # 填充更新时间
            draw_time_text(img1, total_width)
            with Image.new("RGB", (total_width, total_height)) as combined_image:
                # 将截图粘贴到新图像中
                y_offset = 0
                for img in images:
                    combined_image.paste(img, (0, y_offset))
                    y_offset += img.height

                # 保存合并后的图像
                combined_image.save(vb_file)
            img1.close()
    finally:
        # 关闭并删除所有截图文件
        for img_path in img_paths:
            img_path.unlink()


def draw_time_text(img: Image.Image, width: int = 1126):
    draw = ImageDraw.Draw(img)
    font_size = 26
    font = ImageFont.truetype(VB_FONT_PATH, font_size)
    time_text = time.strftime("Updated: %Y-%m-%d %H:%M:%S", time.localtime())
    time_text_width = draw.textlength(time_text, font=font)
    x = width - time_text_width - 10
    draw.text((x, 12), time_text, font=font, fill=(80, 80, 80))


def resize_img_with_right_pixel(img: Image.Image, width: int = 1126):
    new_img = Image.new("RGB", (width, img.height))
    new_img.paste(img, (0, 0))
    # 横向取 img 最右侧像素点，填充到 new_img 的 width - 1 到 width 的像素点
    for x in range(img.width - 50, width):
        for y in range(img.height):
            color = img.getpixel((img.width - 50, y))
            assert color is not None
            new_img.putpixel((x, y), color)
    return new_img



from pathlib import Path

import aiofiles
from PIL import Image


async def save_img(img: Image.Image, path: Path, format: str = "PNG"):
    from io import BytesIO

    buffer = BytesIO()
    img.save(buffer, format=format)
    img_data = buffer.getvalue()
    async with aiofiles.open(path, "wb") as f:
        await f.write(img_data)



import asyncio
from pathlib import Path

from nonebot_plugin_htmlrender import get_new_page
from nonebot_plugin_htmlrender.browser import Page

from .config import GG_FONT_PATH, data_dir

shop_file = data_dir / "shop.png"


async def screenshot_shop_img() -> Path:
    # url = "https://www.fortnite.com/item-shop?lang=zh-Hans"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/116.0.0.0 Safari/537.36",  # noqa: E501
        "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",  # noqa: E501
        "Accept-Encoding": "gzip, deflate",
        "upgrade-insecure-requests": "1",
        "dnt": "1",
        "x-requested-with": "mark.via",
        "sec-fetch-site": "none",
        "sec-fetch-mode": "navigate",
        "sec-fetch-user": "?1",
        "sec-fetch-dest": "document",
        "accept-language": "zh-CN,zh;q=0.9,en-US;q=0.8,en;q=0.7",
    }
    # browser = await get_browser(headless=True)
    # context = await browser.new_context(extra_http_headers=headers)
    async with get_new_page(device_scale_factor=1, extra_http_headers=headers) as page:
        await _screenshot_shop_img(page)
    await add_update_time()
    return shop_file


async def _screenshot_shop_img(page: Page):
    url = "https://fortnite.gg/shop"
    # page.on('requestfailed', lambda request: logger.warning(f'Request failed: {request.url}'))
    await page.add_style_tag(content="* { transition: none !important; animation: none !important; }")
    await page.goto(url)

    async def wait_for_load():
        await page.wait_for_load_state("networkidle", timeout=90000)

    async def scroll_page():
        for _ in range(20):
            await page.evaluate("""() => {
                window.scrollBy(0, document.body.scrollHeight / 20);
            }""")
            await asyncio.sleep(1)  # 等待1秒以加载内容

    await asyncio.gather(wait_for_load(), scroll_page())

    await page.screenshot(path=shop_file, full_page=True)


async def add_update_time():
    await asyncio.to_thread(_add_update_time)


def _add_update_time():
    import time

    from PIL import Image, ImageDraw, ImageFont

    font = ImageFont.truetype(GG_FONT_PATH, 88)
    with Image.open(shop_file) as img:
        draw = ImageDraw.Draw(img)
        # 先填充 rgb(47,49,54) 背景 1280 * 100
        draw.rectangle((0, 0, 1280, 270), fill=(47, 49, 54))
        # 1280 宽，19个数字居中 x 坐标
        time_text = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
        time_text_width = draw.textlength(time_text, font=font)
        x = (1280 - time_text_width) / 2
        draw.text((x, 100), time_text, font=font, fill=(255, 255, 255))
        img.save(shop_file)
